;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;

; "Thermal Key" sprite, by Ersanio
; Uses extra bits: NO
; Read the readme for additional info
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Defines. Use them to customize everything
; to your needs!
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

!Lava		= $0004		;Map16 tile of the "Heating" tile, lava
!Ice		= $0200		;Map16 tile of the "Cooling" tile, ice

!NormalPal	= $00		;Palette of the normal key, CCC bits of YXPPCCCT
!CooledPal	= $06		;Palette of the cooled key, CCC bits of YXPPCCCT
!BlazedPal	= $08		;Palette of the blazing key, CCC bits of YXPPCCCT

!BlazeSFX	= $17		;Blaze SFX number
!BlazeSFXPanel	= $1DFC		;Blaze SFX panel
!CoolSFX	= $07		;Blaze SFX number
!CoolSFXPanel	= $1DF9		;Blaze SFX panel

!KeyState	= $1528		;Keystate RAM address. $00 = normal, $01 = cold, $02 = hot

print "INIT ",pc

INIT:	LDA #$09		;\ Spawn in a stunned state
	STA $14C8,x		;/
	STZ !KeyState,x		;Spawn in a normal temperature
	RTL

print "MAIN ",pc
MAIN:
	PHB
	PHK
	PLB
	JSR Key
	PLB
	RTL

Key:	JSR Graphics		;Draw graphics on screen
	JSR block_interact	;Detect block below the sprite

	REP #$20		;\
	CMP #!Lava		;/If block is not lava, branch
	BNE +			;\
	SEP #$20		; |
	LDA !KeyState,x		; |
	CMP #$02		; |Start burning only if key isn't burning already
	BEQ +			; |
	LDA #$02		; |
	STA !KeyState,x		; |
	JSR SUB_SMOKE		; |
	LDA #!BlazeSFX		; |
	STA !BlazeSFXPanel	;/
	RTS

+	REP #$20		;\
	CMP #!Ice		; |If block is not ice, return (there are no other blocks)
	BNE +			;/
	SEP #$20		;\
	LDA !KeyState,x		; |
	CMP #$01		; |Start freezing only if key isn't frozen already
	BEQ +			; |
	LDA #$01		; |
	STA !KeyState,x		; |
	LDA #!CoolSFX		; |
	STA !CoolSFXPanel	; |
	JSR SUB_SMOKE		;/
+	SEP #$20		;\
	RTS			;/

PalTbl:	db !NormalPal,!CooledPal,!BlazedPal

Graphics:
	JSL GET_DRAW_INFO
	LDA $00
	STA $0300,y
	LDA $01
	STA $0301,y

	LDA $0303,y
	AND #$F1
	PHX
	PHY
	LDY !KeyState,x
	ORA PalTbl,y
	PLY
	PLX
	STA $0303,y
	LDY #$02
	LDA #$00
	JSL $01B7B3

	JSR Particles		;Particle effects for the 2 different states

	RTS

Particles:
	LDA !KeyState,x		;Key state
	CMP #$01		;\If frozen,
	BEQ FrozenParticle	;/ice particles
	CMP #$02		;\Else, if burning
	BEQ BlazeParticle	;/fire particles
Return:	RTS			;Else, return

BlazeParticle:
	LDA $D8,x
	PHA
	SEC
	SBC #$08
	STA $D8,x
	LDA $13
	AND #$03	;Every 3 frames, generate fire particle
	ORA $186C,x
	ORA $9D
	BNE +
	JSL $0285E4
+	PLA
	STA $D8,x
	RTS

FrozenParticle:
	LDA $13
	AND #$07	;Every 7 frames, generate ice particle
	ORA $9D

	PHK
	PEA.w .jslrtsreturn-1
	PEA $90B8
	JML $01B152
.jslrtsreturn
	RTS

GET_DRAW_INFO:		STZ $186C,x
			STZ $15A0,x
			LDA $E4,x
			CMP $1A
			LDA $14E0,x
			SBC $1B
			BEQ ON_SCREEN_X
			INC $15A0,x

ON_SCREEN_X:		LDA $14E0,x
			XBA
			LDA $E4,x
			REP #$20
			SEC
			SBC $1A
			CLC
			ADC #$0040
			CMP #$0180
			SEP #$20
			ROL A
			AND #$01
			STA $15C4,x
			BNE INVALID

			PHB
			PHK
			PLB
			LDY #$00
			LDA $1662,x
			AND #$20
			BEQ ON_SCREEN_LOOP
			INY
ON_SCREEN_LOOP:		LDA $D8,x
			CLC
			ADC SPR_T1,y
			PHP
 			CMP $1C
			ROL $00
			PLP
			LDA $14D4,x
			ADC #$00
			LSR $00
			SBC $1D
			BEQ ON_SCREEN_Y
			LDA $186C,x
			ORA SPR_T2,y
			STA $186C,x
ON_SCREEN_Y:		DEY
			BPL ON_SCREEN_LOOP

			LDY $15EA,x
			LDA $E4,x
			SEC
			SBC $1A
			STA $00
			LDA $D8,x
			SEC
			SBC $1C
			STA $01
			PLB
			RTL

INVALID:		REP #$20
			PLA
			PLY
			PLA
			PHY
			PHA
			SEP #$20
			RTL

SPR_T1:			db $0C,$1C
SPR_T2:			db $01,$02


;thanks p4plus2 for this routine

block_interact:
	REP #$30
	JSR get_pos
	LDA $05
	CLC
	ADC #$0020
	STA $05
	LDA $07
	CLC
	ADC #$0008
	STA $07
	JSR get_block
;A has block now
	SEP #$30
	LDX $15E9
	RTS
	
get_pos:
	SEP #$30
	LDX $15E9
	LDA $D8,x
	STA $05 
	LDA $14D4,x
	STA $06  
	LDA $E4,x
	STA $07  
	LDA $14E0,x
	STA $08  
	STA $09
	REP #$30
	RTS
	
get_block:
	LDA $1925
	AND #$00FF
	ASL
	TAX
	LDA $00BDA8,x
	STA $0A
	LDA $00BE28,x
	STA $0D
	SEP #$20
	LDA #$00
	STA $0C
	STA $0F
	XBA
	LDA $08
	ASL
	CLC
	ADC $08
	REP #$20
	TAY
	LDA [$0A],y
	STA $6B
	LDA [$0D],y
	STA $6E
	SEP #$30
	LDA #$7E
	STA $6D
	INC
	STA $70
	REP #$30
	LDA $05
	AND #$01F0
	STA $00
	LDA $07
	AND #$00F0
	LSR #4
	CLC
	ADC $00
	TAY
	SEP #$20
	LDA [$6E],y
	XBA
	LDA [$6B],y
	REP #$20
	RTS

SUB_SMOKE:		LDY #$03	; \ find a free slot to display effect
FINDFREE:		LDA $17C0,y	;  |
			BEQ FOUNDONE	;  |
			DEY		;  |
			BPL FINDFREE	;  |
			RTS		; / return if no slots open

FOUNDONE:		LDA #$01	; \ set effect graphic to smoke graphic
			STA $17C0,y	; /
			LDA #$1B	; \ set time to show smoke
			STA $17CC,y	; /
			LDA $D8,x	; \ smoke y position = generator y position
			STA $17C4,y	; /
			LDA $E4,x	; \ load generator x position and store it for later
			STA $17C8,y	; /
			RTS